package com.graywolf336.jail.listeners;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.FoodLevelChangeEvent;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.player.AsyncPlayerChatEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerKickEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.player.PlayerRespawnEvent;

import com.graywolf336.jail.JailMain;
import com.graywolf336.jail.JailManager;
import com.graywolf336.jail.Util;
import com.graywolf336.jail.beans.AnyCell;
import com.graywolf336.jail.beans.Cell;
import com.graywolf336.jail.beans.Jail;
import com.graywolf336.jail.beans.Prisoner;
import com.graywolf336.jail.beans.Stick;
import com.graywolf336.jail.enums.Lang;
import com.graywolf336.jail.enums.Settings;
import com.graywolf336.jail.events.PrePrisonerJailedByJailStickEvent;
import com.graywolf336.jail.events.PrisonerDeathEvent;
import com.graywolf336.jail.interfaces.ICell;

public class PlayerListener implements Listener {
    private JailMain pl;
    private HashMap<UUID, Long> clicks;

    public PlayerListener(JailMain plugin) {
        this.pl = plugin;
        this.clicks = new HashMap<UUID, Long>();
    }

    @EventHandler(ignoreCancelled=true)
    public void jailOrCellCreation(PlayerInteractEvent event) {
        if(event.getAction() == Action.RIGHT_CLICK_AIR || event.getAction() == Action.RIGHT_CLICK_BLOCK) {
            Player p = event.getPlayer();
            Location loc = event.getClickedBlock() == null ? p.getLocation() : event.getClickedBlock().getLocation();
            JailManager jm = pl.getJailManager();

            if(p.getInventory().getItemInMainHand().isSimilar(Util.getWand())) {
                if(jm.isCreatingSomething(p.getName())) {
                    
                    long current = System.currentTimeMillis();
                    if(clicks.containsKey(p.getUniqueId())) {
                        if(current - clicks.get(p.getUniqueId()) > 500) {
                            clicks.put(p.getUniqueId(), current);
                        }else {
                            pl.debug("Prevented a double click for " + p.getName() + "!");
                            return;
                        }
                    }else {
                        clicks.put(p.getUniqueId(), current);
                    }
                    
                    if(jm.isCreatingAJail(p.getName())) {
                        pl.debug("Stepping into creating a jail.");
                        jm.getJailCreationSteps().step(jm, p, jm.getJailCreationPlayer(p.getName()), loc);
                    }else if(jm.isCreatingACell(p.getName())) {
                        pl.debug("Stepping into creating a cell.");
                        jm.getCellCreationSteps().step(jm, p, jm.getCellCreationPlayer(p.getName()), loc);
                    }
                }
            }
        }
    }

    @EventHandler(ignoreCancelled=true, priority = EventPriority.LOW)
    public void chatting(AsyncPlayerChatEvent event) {
        if(pl.getJailManager().isPlayerJailed(event.getPlayer().getUniqueId())) {
            if(pl.getJailManager().getPrisoner(event.getPlayer().getUniqueId()).isMuted()) {
                event.setCancelled(true);
                event.getPlayer().sendMessage(Lang.MUTED.get());
            }
        }

        //If the config has receive messages set to false, let's remove all the prisoners
        //from getting the chat messages.
        if(!pl.getConfig().getBoolean(Settings.RECIEVEMESSAGES.getPath())) {
            if(pl.inDebug()) pl.getLogger().info("Debug - There are " + event.getRecipients().size() + " players getting the message before.");
            Set<Player> rec = new HashSet<Player>(event.getRecipients());

            for(Jail j : pl.getJailManager().getJails())
                for(Prisoner p : j.getAllPrisoners().values())
                    rec.remove(pl.getServer().getPlayer(p.getUUID()));

            event.getRecipients().clear();
            event.getRecipients().addAll(rec);
            if(pl.inDebug()) pl.getLogger().info("Debug - There are now " + event.getRecipients().size() + " players getting the message.");
        }
    }

    @EventHandler(priority = EventPriority.HIGHEST)
    public void checkForOfflineJailStuff(PlayerJoinEvent event) {
        //Let's check if the player is jailed
        if(pl.getJailManager().isPlayerJailed(event.getPlayer().getUniqueId())) {
            //Get the prisoner object
            Jail j = pl.getJailManager().getJailPlayerIsIn(event.getPlayer().getUniqueId());

            if(!j.isEnabled()) {
                event.getPlayer().kickPlayer(Lang.WORLDUNLOADEDKICK.get());
                pl.getLogger().warning(j.getName() + " is located in a world which is unloaded and " + event.getPlayer().getName() + " (" + event.getPlayer().getUniqueId() + ") tried to join while jailed in it.");
                return;
            }

            Prisoner p = j.getPrisoner(event.getPlayer().getUniqueId());
            //update their last known username when they login
            p.setLastKnownName(event.getPlayer().getName());

            //Check if they're offline pending, as if this is true then they were jailed offline
            if(p.isOfflinePending()) {
                if(p.getRemainingTime() == 0L) {
                    //If their remaining time is 0, let's unjail them
                    pl.getPrisonerManager().schedulePrisonerRelease(p);
                }else if(p.isToBeTransferred()) {
                    Cell c = j.getCellPrisonerIsIn(event.getPlayer().getUniqueId());

                    //If the player is not jailed in a cell, teleport them to the jail's in
                    if(c == null) {
                        p.setTeleporting(true);
                        event.getPlayer().teleport(j.getTeleportIn());
                        p.setTeleporting(false);
                    }else {
                        //If they are in a cell, teleport them into that cell
                        p.setTeleporting(true);
                        event.getPlayer().teleport(c.getTeleport());
                        p.setTeleporting(false);
                    }

                    p.setToBeTransferred(false);
                } else {
                    //Their remaining time isn't 0 so let's proceed with jailing of the prisoner
                    pl.getPrisonerManager().jailPlayer(event.getPlayer().getUniqueId());
                }
            }

            //Add the scoreboard to them if it is enabled
            if(pl.getConfig().getBoolean(Settings.SCOREBOARDENABLED.getPath())) {
                pl.getScoreBoardManager().addScoreBoard(event.getPlayer(), p);
            }

            //if we are ignoring a prisoner's sleeping state, then let's set that
            if(pl.getConfig().getBoolean(Settings.IGNORESLEEPINGSTATE.getPath())) {
                event.getPlayer().setSleepingIgnored(true);
            }
        }
    }

    @EventHandler
    public void notifyUpdate(PlayerJoinEvent event) {
        if(pl.getConfig().getBoolean(Settings.UPDATENOTIFICATIONS.getPath())) {
            if(event.getPlayer().isOp() && pl.getUpdate().isAvailable()) {
                event.getPlayer().sendMessage(ChatColor.BLUE + "" + ChatColor.BOLD + "An update for Jail is available: " + pl.getUpdate().getNewVersion());
                event.getPlayer().sendMessage(ChatColor.BLUE + "" + ChatColor.BOLD + pl.getUpdate().getFileUrl());
            }
        }
    }

    @EventHandler
    public void handleGoingOffline(PlayerQuitEvent event) {
        if(pl.getJailManager().isPlayerJailed(event.getPlayer().getUniqueId())) {
            //Remove the scoreboard to them if it is enabled
            if(pl.getConfig().getBoolean(Settings.SCOREBOARDENABLED.getPath())) {
                pl.getScoreBoardManager().removeScoreBoard(event.getPlayer());
            }
        }
    }

    @EventHandler(ignoreCancelled=true)
    public void handleGettingKicked(PlayerKickEvent event) {
        if(pl.getJailManager().isPlayerJailed(event.getPlayer().getUniqueId())) {
            //Remove the scoreboard to them if it is enabled
            if(pl.getConfig().getBoolean(Settings.SCOREBOARDENABLED.getPath())) {
                pl.getScoreBoardManager().removeScoreBoard(event.getPlayer());
            }
        }
        
        clicks.remove(event.getPlayer().getUniqueId());
    }

    @EventHandler(priority = EventPriority.LOW)
    public void foodControl(FoodLevelChangeEvent event) {
        if(pl.getConfig().getBoolean(Settings.FOODCONTROL.getPath())) {
            if(pl.getJailManager().isPlayerJailed(event.getEntity().getUniqueId())) {
                int min = pl.getConfig().getInt(Settings.FOODCONTROLMIN.getPath());
                int max = pl.getConfig().getInt(Settings.FOODCONTROLMAX.getPath());

                if (event.getFoodLevel() <  min) {
                    event.setFoodLevel(min);
                }else {
                    event.setFoodLevel(max);
                }
            }
        }
    }

    @EventHandler
    public void onPlayerDeath(PlayerDeathEvent event) {
        if(pl.getJailManager().isPlayerJailed(event.getEntity().getUniqueId())) {
            Jail j = pl.getJailManager().getJailPlayerIsIn(event.getEntity().getUniqueId());

            PrisonerDeathEvent prisonerEvent = new PrisonerDeathEvent(event, j, j.getCellPrisonerIsIn(event.getEntity().getUniqueId()), j.getPrisoner(event.getEntity().getUniqueId()), event.getEntity());
            pl.getServer().getPluginManager().callEvent(prisonerEvent);
        }
    }

    @EventHandler(priority = EventPriority.HIGHEST)
    public void onPlayerRespawn(PlayerRespawnEvent event) {
        if(pl.getJailManager().isPlayerJailed(event.getPlayer().getUniqueId())) {
            Jail j = pl.getJailManager().getJailPlayerIsIn(event.getPlayer().getUniqueId());

            if(j.isJailedInACell(event.getPlayer().getUniqueId())) {
                event.setRespawnLocation(j.getCellPrisonerIsIn(event.getPlayer().getUniqueId()).getTeleport());
            }else {
                event.setRespawnLocation(j.getTeleportIn());
            }
        }
    }

    @EventHandler(priority = EventPriority.LOWEST, ignoreCancelled=true)
    public void jailStickHandling(EntityDamageByEntityEvent event) {
        if(pl.getConfig().getBoolean(Settings.JAILSTICKENABLED.getPath())) {
            //If the damager and the entity getting damage is not a player,
            //we don't want to handle it in this method
            if(!(event.getDamager() instanceof Player) || !(event.getEntity() instanceof Player)) return;

            Player attacker = (Player) event.getDamager();
            Player player = (Player) event.getEntity();
            
            //They aren't using the jail stick, they need to toggle themselves
            if(!pl.getJailStickManager().isUsingJailStick(attacker.getUniqueId())) return;
            
            //They don't have a valid jail stick type
            if(!pl.getJailStickManager().isValidStick(attacker.getInventory().getItemInMainHand().getType())) return;
            
            //They don't have permission for that type of jail stick
            if(!attacker.hasPermission("jail.usejailstick." + attacker.getInventory().getItemInMainHand().getType().toString().toLowerCase())) return;
            
            //The player they hit is already jailed.
            if(pl.getJailManager().isPlayerJailed(player.getUniqueId())) {
            	attacker.sendMessage(Lang.ALREADYJAILED.get(player.getName()));
            	return;
            }

            //The player can't be jailed due to permissions.
            if(player.hasPermission("jail.cantbejailed")) {
                attacker.sendMessage(Lang.CANTBEJAILED.get());
                return;
            }
            
            //Get the stick the attacker (police) is using
            Stick s = pl.getJailStickManager().getStick(attacker.getInventory().getItemInMainHand().getType());

            //Check if the player can resist arrest via health with this stick
            pl.debug("Jail stick health resist check: " + "(" + s.getHealth() + "," + player.getHealth() + ") - " + (s.getHealth() <= player.getHealth() && s.getHealth() != -1));
            if(s.getHealth() < player.getHealth() && s.getHealth() != -1) {
            	attacker.sendMessage(Lang.RESISTEDARRESTJAILER.get(player.getName()));
                player.sendMessage(Lang.RESISTEDARRESTPLAYER.get(attacker.getName()));
            	return;
            }
            
            //The player can be jailed.
            Prisoner p = new Prisoner(player.getUniqueId().toString(), player.getName(),
                    pl.getConfig().getBoolean(Settings.AUTOMATICMUTE.getPath()),
                    s.getTime(), attacker.getName(), s.getReason());

            PrePrisonerJailedByJailStickEvent jEvent = new PrePrisonerJailedByJailStickEvent(
                    pl.getJailManager().getJail(s.getJail()), new AnyCell(), p, player, attacker.getName(), s);

            pl.getServer().getPluginManager().callEvent(jEvent);

            if(jEvent.isCancelled()) {
                if(jEvent.getCancelledMessage().isEmpty())
                    attacker.sendMessage(Lang.CANCELLEDBYANOTHERPLUGIN.get(player.getName()));
                else
                    attacker.sendMessage(jEvent.getCancelledMessage());
            }else {
                //recall data from the event
                Jail j = jEvent.getJail();
                ICell c = jEvent.getCell();
                p = jEvent.getPrisoner();
                player = jEvent.getPlayer();

                //Player is not online
                if(player == null) {
                    attacker.sendMessage(Lang.OFFLINEJAIL.get(new String[] { p.getLastKnownName(), String.valueOf(p.getRemainingTimeInMinutes()) }));
                }else {
                    //Player *is* online
                    attacker.sendMessage(Lang.ONLINEJAIL.get(new String[] { p.getLastKnownName(), String.valueOf(p.getRemainingTimeInMinutes()) }));
                }

                try {
                    pl.getPrisonerManager().prepareJail(j, c, player, p);
                } catch (Exception e) {
                    attacker.sendMessage(ChatColor.RED + e.getMessage());
                }
            }
        }
    }
}
